@@ -3,7 +3,7 @@ package Image::Libpuzzle;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 require XSLoader;
 XSLoader::load('Image::Libpuzzle', $VERSION);
 
@@ -17,8 +17,8 @@ our $PUZZLE_CVEC_SIMILARITY_HIGH_THRESHOLD  = Image::Libpuzzle->PUZZLE_CVEC_SIMI
 our $PUZZLE_CVEC_SIMILARITY_LOW_THRESHOLD   = Image::Libpuzzle->PUZZLE_CVEC_SIMILARITY_LOW_THRESHOLD;
 our $PUZZLE_CVEC_SIMILARITY_LOWER_THRESHOLD = Image::Libpuzzle->PUZZLE_CVEC_SIMILARITY_LOWER_THRESHOLD;
 
-# uses unpack and $self accessor to get signature directly from the internal cvec
-sub signature_as_string {
+# uses unpack as bin to char and $self accessor to get signature directly from the internal cvec
+sub signature_as_char_string {
   my $self = shift;
   my @sig  = unpack("C*", $self->get_signature());
   my $sig  = q{};
@@ -29,10 +29,33 @@ sub signature_as_string {
 }
 
 # returns an ARRAY ref of all ngrams, of L-R sliding window of size $ngram_size
-sub signature_as_ngrams {
+sub signature_as_char_ngrams {
   my $self       = shift;
   my $ngram_size = shift || $DEFAULT_NGRAM_SIZE;
-  my $sig        = $self->signature_as_string;
+  my $sig        = $self->signature_as_char_string;
+  my @ngrams     = ();
+  for (my $i = 0; $i <= length($sig) - $ngram_size; $i++) {
+    push @ngrams, substr($sig, $i, $ngram_size);
+  }
+  return \@ngrams;
+}
+
+# uses unpack as bin to hex and $self accessor to get signature directly from the internal cvec
+sub signature_as_hex_string {
+  my $self = shift;
+  my @sig  = unpack("H*", $self->get_signature());
+  my $sig  = q{};
+  foreach my $i (@sig) {
+    $sig .= sprintf("%s", $i);
+  }
+  return $sig;
+}
+
+# returns an ARRAY ref of all ngrams, of L-R sliding window of size $ngram_size
+sub signature_as_hex_ngrams {
+  my $self       = shift;
+  my $ngram_size = shift || $DEFAULT_NGRAM_SIZE;
+  my $sig        = $self->signature_as_hex_string;
   my @ngrams     = ();
   for (my $i = 0; $i <= length($sig) - $ngram_size; $i++) {
     push @ngrams, substr($sig, $i, $ngram_size);
@@ -83,19 +106,19 @@ Image::Libpuzzle - Perl interface to libpuzzle.
      $sig2 = $p2->fill_cvec_from_file($pic2);
 
      # stringify sig1
-     my $string1 = $p1->signature_as_string;
+     my $string1 = $p1->signature_as_char_string;
      print qq{$string1\n};
 
      # stringify sig2
-     my $string2 = $p2->signature_as_string;
+     my $string2 = $p2->signature_as_char_string;
      print qq{$string2\n};
 
      # generate a "document" of ngrams from sig1
-     my $words1_ref = $p1->signature_as_ngrams; # defaults to $ngram size of $Image::Libpuzzle::DEFAULT_NGRAM_SIZE
+     my $words1_ref = $p1->signature_as_char_ngrams; # defaults to $ngram size of $Image::Libpuzzle::DEFAULT_NGRAM_SIZE
      print join ' ', @$words1_ref;
 
      # generate a "document" of ngrams from sig2
-     my $words2_ref = $p2->signature_as_ngrams(6); # example overriding $Image::Libpuzzle::DEFAULT_NGRAM_SIZE 
+     my $words2_ref = $p2->signature_as_char_ngrams(6); # example overriding $Image::Libpuzzle::DEFAULT_NGRAM_SIZE 
      print join ' ', @$words2_ref;
 
      # print Euclidean length of sig1
@@ -147,7 +170,7 @@ C<vector_normalized_distance>.
 
 C<Image::Libpuzzle> provides two methods for generating signatures in a
 printable form that may be used to deal with signatures in a more printable way,
-C<signature_as_string> and C<signature_as_ngrans>. See below for more details.
+C<signature_as_char_string> and C<signature_as_ngrans>. See below for more details.
 
 =head2 Comparing Millions of Images
 
@@ -156,7 +179,7 @@ question:
 
 L<http://stackoverflow.com/questions/9703762/libpuzzle-indexing-millions-of-pictures>
 
-The C<Image::Libpuzzle::signature_as_ngrams> methods may be used to generate
+The C<Image::Libpuzzle::signature_as_char_ngrams> methods may be used to generate
 ngrams (words of size N) for use with the oft suggested approach to searching
 for similar images in a database of signatures.
 
@@ -167,11 +190,12 @@ but may be added in the future if there is demand.
 
 =head1 XS METHODS AND SUBROUTINES
 
-=head2 new
+=head2 C<new()>
 
-Constructor, returns a C<Image::Libpuzzle> reference.
+Constructor, returns a C<Image::Libpuzzle> reference. Use C<set_*> methods to change the
+default values outlined in C<man puzzle_set(3)>.
 
-=head2 C<get_cvec>
+=head2 C<get_cvec()>
 
 Returns a Image::Libpuzzle::Cvec reference, currently one can't do much with
 this.
@@ -180,9 +204,11 @@ this.
 
 Generates the signature for the given file.
 
-=head2 C<get_signature>
+=head2 C<get_signature()>
 
-Returns the signature in an unprintable form.
+Returns the signature in the original form.
+
+Assumes C<fill_cvec_from_file> has been called on a valid image already.
 
 =head2 C<set_lambdas($integer)>
 
@@ -192,7 +218,7 @@ image.
 The default is set in puzzle.h is 9; i.e., by default, pictures are divided in 9
 x 9 blocks.
 
-C<puzzle_set(3)> says,
+C<man puzzle_set(3)> says,
 
 For large databases, for complex images, for images with a lot of text or for
 sets of near-similar images, it might be better to raise that value to 11 or
@@ -221,7 +247,7 @@ The lowest acceptable value is 1.0.
 Wrapper around Libpuzzle's function. Sets the size of the samples. Used in
 conjunction with C<set_lambdas> to get more or less precise signatures.
 
-C<puzzle_set(3)> says,
+C<man puzzle_set(3)> says,
 
 The "p ratio" determines the size of that zone. The default is 2.0, and that
 ratio mimics the behavior that is described in the reference algorithm.
@@ -230,7 +256,7 @@ ratio mimics the behavior that is described in the reference algorithm.
 
 Wrapper around Libpuzzle's function.
 
-C<puzzle_set(3)> says,
+C<man puzzle_set(3)> says,
 
 In order to avoid CPU starvation, pictures won't be processed if their width or
 height is larger than 3000 pixels.
@@ -239,13 +265,13 @@ height is larger than 3000 pixels.
 
 Wrapper around Libpuzzle's function.
 
-See L<set_max_width>.
+See C<set_max_width>.
 
 =head2 C<set_noise_cutoff($integer)>
 
 Wrapper around Libpuzzle's function.
 
-C<puzzle_set(3)> says,
+C<man puzzle_set(3)> says,
 
 The noise cutoff defaults to 2. If you raise that value, more zones with little
 difference of intensity will be considered as similar.
@@ -257,7 +283,7 @@ change this.
 
 Wrapper around Libpuzzle's function.
 
-C<puzzle_set(3)> says,
+C<man puzzle_set(3)> says,
 
 By default, featureless borders of the original image are ignored. The size
 of each border depends on the sum of absolute values of differences between
@@ -279,13 +305,13 @@ will ever be shaved.
 
 Wrapper around Libpuzzle's function.
 
-See L<set_autocrop> for details.
+See C<set_autocrop> for details.
 
 =head2 C<set_max_cropping_ratio($double)>
 
 Wrapper around Libpuzzle's function.
 
-See L<set_autocrop> for details.
+See C<set_autocrop> for details.
 
 =head2 C<vector_euclidean_length()>
 
@@ -299,6 +325,15 @@ Returns the computed distance between two C<Image::Libpuzzle> instances.
 
  my $distance = $instance1->vector_normalized_distance($instance2);
 
+NOTE: internally, "fix_for_texts" is set to 1; there is currently no way to toggle this behavior.
+
+According to C<man 3 libpuzzle>: 
+
+If the fix_for_texts of puzzle_vector_normalized_distance() is 1 , a fix
+is applied to the computation in order to deal with bitmap pictures that
+contain text. That fix is recommended, as it allows using the same
+threshold for that kind of picture as for generic pictures.
+
 =head2 C<is_similar(Image::Libpuzzle $instance2)>
 
 Convenience methods, compares images using C<PUZZLE_CVEC_SIMILARITY_THRESHOLD>
@@ -342,16 +377,18 @@ considering two images as similar. Used by C<is_most_similar>.
 
 =head1 Pure Perl METHODS AND SUBROUTINES
 
-=head2 C<signature_as_string()>
+=head2 C<signature_as_char_string()>
 
 Returns a stringified version of the signature. The string is generated by
 unpack'ing into an array of ASCII characters (C*). Before the array of character
 codes is joined into a string, they are padded. For example, 1 turns into 001;
 25 turns into 025; 211 remains the same.
 
-=head2 C<signature_as_ngrams()>
+Assumes C<fill_cvec_from_file> has been called on a valid image already.
 
-Takes the output of C<signature_as_string> and returns an ARRAY ref of C<words>
+=head2 C<signature_as_char_ngrams()>
+
+Takes the output of C<signature_as_char_string> and returns an ARRAY ref of C<words>
 of size C<$ngram_size>. The default, C<$DEFAULT_NGRAM_SIZE> is set to 10. An
 optional argument may be passed to override this default.
 
@@ -361,6 +398,21 @@ described in the following link:
 L<http://stackoverflow.com/questions/9703762/libpuzzle-indexing-millions-of-pict
 ures>
 
+Assumes C<fill_cvec_from_file> has been called on a valid image already.
+
+=head2 C<signature_as_hex_string()>
+
+Returns a stringified version of the signature. The string is generated by
+unpack'ing into an array of ASCII hexacters (C*). Before the array of hexacter
+codes is joined into a string, they are padded. For example, 1 turns into 001;
+25 turns into 025; 211 remains the same.
+
+=head2 C<signature_as_hex_ngrams()>
+
+Takes the output of C<signature_as_hex_string> and returns an ARRAY ref of C<words>
+of size C<$ngram_size>. The default, C<$DEFAULT_NGRAM_SIZE> is set to 10. An
+optional argument may be passed to override this default.
+
 =head1 ENVIRONMENT
 
 This module assumes that libpuzzle is installed and puzzle.h is able to be found
@@ -398,7 +450,7 @@ Please report them via L<https://github.com/estrabd/p5-puzzle-xs/issues>.
 
 =head1 AUTHOR
 
-Brett Estrade <estrabd@gmail.com>
+B. Estrade <estrabd@gmail.com>
 
 =head2 THANK YOU
 
